######################################################################

set MonitorIndicatorTable {
	{ 0 \#00a7ff \#002266 } { 1 \#00a7ff \#002266 }
	{ 2 \#00a7ff \#002266 } { 3 \#00a7ff \#002266 }
	{ 4 \#00a7ff \#002266 } { 5 \#00a7ff \#002266 }
	{ 7 \#00a7ff \#002266 }
	{ 10 \#49dc00 \#0a4c00 } { 13 \#49dc00 \#0a4c00 }
	{ 19 \#49dc00 \#0a4c00 } { 26 \#49dc00 \#0a4c00 }
	{ 37 \#49dc00 \#0a4c00 } { 51 \#49dc00 \#0a4c00 }
	{ 71 \#49dc00 \#0a4c00 }
	{ 100 \#ebd000 \#514800 } { 138 \#ebd000 \#514800 }
	{ 193 \#ebd000 \#514800 } { 268 \#ebd000 \#514800 }
	{ 372 \#ebd000 \#514800 } { 517 \#ebd000 \#514800 }
	{ 719 \#ebd000 \#514800 }
	{ 1000 \#eb9c00 \#513300 } { 1389 \#eb9c00 \#513300 }
	{ 1930 \#eb9c00 \#513300 } { 2682 \#eb9c00 \#513300 }
	{ 3727 \#eb9c00 \#513300 } { 5179 \#eb9c00 \#513300 }
	{ 7196 \#eb9c00 \#513300 }
	{ 10000 \#e10000 \#510000 } { 13894 \#e10000 \#510000 }
	{ 19306 \#e10000 \#510000 } { 26827 \#e10000 \#510000 }
	{ 37275 \#e10000 \#510000 } { 51794 \#e10000 \#510000 }
	{ 71968 \#e10000 \#510000 }
}

set MonitorSendRate 0
set MonitorRecvRate 0

proc Monitor_Indicator { f unit } {
	global UpdatePacketEvent
	global MonitorIndicatorTable
	global MonitorSendRate MonitorRecvRate

	frame $f
	label $f.inLabel -text "In:"
	label $f.outLabel -text "Out:"

	frame $f.inIndicators -relief sunken -borderwidth 2
	frame $f.outIndicators -relief sunken -borderwidth 2

	set i 0
	foreach ind $MonitorIndicatorTable {
		label $f.inIndicators.i$i -relief raised \
			-width 0 -padx $unit -borderwidth 1 \
			-background [lindex $ind 2]
		label $f.outIndicators.i$i -relief raised \
			-width 0 -padx $unit -borderwidth 1 \
			-background [lindex $ind 2]
		pack $f.inIndicators.i$i -side left
		pack $f.outIndicators.i$i -side left
		incr i
	}
	label $f.inRate -width 8 -relief sunken -anchor e \
		-textvariable MonitorRecvRate
	label $f.outRate -width 8 -relief sunken -anchor e \
		-textvariable MonitorSendRate

	grid $f.inLabel -column 0 -row 0 -sticky e
	grid $f.outLabel -column 0 -row 1 -sticky e
	grid $f.inIndicators -column 1 -row 0 -sticky w
	grid $f.outIndicators -column 1 -row 1 -sticky w
#	grid $f.inRate -column 2 -row 0 -sticky ew
#	grid $f.outRate -column 2 -row 1 -sticky ew

	PPxP_RegisterUpdateEvent \
		[list Monitor_IndicatorUpdate $f.inIndicators $f.outIndicators]
}

proc Monitor_IndicatorUpdate { fin fout } {
	global PPxPInfo MonitorIndicatorTable
	global MonitorSendRate MonitorRecvRate

	set srate $PPxPInfo(send.nrate)
	set rrate $PPxPInfo(recv.nrate)

	if { $rrate != $MonitorRecvRate } {
		set i 0
		foreach t $MonitorIndicatorTable {
			if { $rrate > [lindex $t 0] } {
				$fin.i$i configure -background [lindex $t 1]
			} else {
				$fin.i$i configure -background [lindex $t 2]
			}
			incr i
		}
	}
	if { $srate != $MonitorSendRate } {
		set i 0
		foreach t $MonitorIndicatorTable {
			if { $srate > [lindex $t 0] } {
				$fout.i$i configure -background [lindex $t 1]
			} else {
				$fout.i$i configure -background [lindex $t 2]
			}
			incr i
		}
	}
	set MonitorSendRate $srate
	set MonitorRecvRate $rrate
}

######################################################################

proc Monitor_Time { frame ppxp } {
	global PPxPEnv
	global MonitorConnectTime MonitorIdleTime UpdatePacketEvent
	global MonitorStatus

	frame $frame
	frame $frame.connect
	label $frame.connect.label -text "OnLine:"
	label $frame.connect.time -width 8 -anchor e -relief sunken \
		-textvariable MonitorConnectTime
	pack $frame.connect.label $frame.connect.time -side left

	frame $frame.idle
	label $frame.idle.label -text "Idle:"
	label $frame.idle.time -width 5 -anchor e -relief sunken \
		-textvariable MonitorIdleTime
	button $frame.idle.extend -image ImageDRight -padx 0 \
		-command [list Monitor_IdleExtend $ppxp]
	button $frame.idle.pause -image ImagePause -padx 0 \
		-command [list Monitor_IdlePause $ppxp]
	pack $frame.idle.label $frame.idle.time -side left
	pack $frame.idle.pause $frame.idle.extend -side right -padx 0

	pack $frame.connect $frame.idle -side top -fill x -expand yes

	proc Monitor_IdlePauseRelief_$frame { name elem op } \
		[list Monitor_IdlePauseRelief $frame.idle.pause]
	trace variable PPxPEnv(IDLE.MODE) w Monitor_IdlePauseRelief_$frame

	set MonitorConnectTime "00:00:00"
	set MonitorIdleTime "00:00"
	Monitor_IdleState [list $frame.idle.pause $frame.idle.extend]

	PPxP_RegisterUpdateEvent Monitor_TimeUpdate
	PPxP_RegisterStatusEvent \
		[list Monitor_IdleState \
			 [list $frame.idle.pause $frame.idle.extend]]
}

proc Monitor_TimeUpdate {} {
	global PPxPInfo
	global MonitorConnectTime MonitorIdleTime

	set t $PPxPInfo(connect)
	set h [expr $t/3600]
	set m [expr ($t%3600)/60]
	set s [expr $t%60]
	set MonitorConnectTime [format "%02d:%02d:%02d" $h $m $s]

	set t $PPxPInfo(idle)
	set h [expr $t/3600]
	set m [expr ($t%3600)/60]
	set s [expr $t%60]
	if { $h != 0 } {
		set MonitorIdleTime "--:--"
	} else {
		set MonitorIdleTime [format "%02d:%02d" $m $s]
	}
}

proc Monitor_IdleExtend { ppxp } {
	global PPxPInfo
	set t [expr $PPxPInfo(idle) + 60]
	$ppxp set IDLE.TIME $t
}

set MonitorIdleModePrev "pause"
proc Monitor_IdlePause { ppxp } {
	global PPxPEnv MonitorIdleModePrev
	if { $PPxPEnv(IDLE.MODE) == "pause" } {
		if { $MonitorIdleModePrev == "pause" } {
			$ppxp set IDLE.MODE bidirectional
		} else {
			$ppxp set IDLE.MODE $MonitorIdleModePrev
		}
	} else {
		set MonitorIdleModePrev $PPxPEnv(IDLE.MODE)
		$ppxp set IDLE.MODE pause
	}
}

proc Monitor_IdlePauseRelief { button } {
	global PPxPEnv
	if { $PPxPEnv(IDLE.MODE) == "pause" } {
		$button configure -relief sunken
	} else {
		$button configure -relief raised
	}
}

proc Monitor_IdleState { buttons } {
	global PPxPStatusString
	if { $PPxPStatusString == "network" } {
		set s normal
	} else {
		set s disabled
	}
	foreach b $buttons {
		$b configure -state $s
	}
}

######################################################################

proc Monitor_Auto { frame ppxp } {
	global UpdatePacketEvent MonitorAutoFlag
	checkbutton $frame -text "Auto" -variable MonitorAutoFlag \
		-command [list Monitor_AutoCommand $ppxp]
	PPxP_RegisterUpdateEvent {
		global PPxPInfo MonitorAutoFlag
		if { $PPxPInfo(mode) == "auto" } {
			set MonitorAutoFlag 1
		} else {
			set MonitorAutoFlag 0
		}
	}
}

proc Monitor_AutoCommand { ppxp } {
	global PPxPInfo
	if { $PPxPInfo(mode) == "auto" } {
		$ppxp auto off
	} else {
		$ppxp auto
	}
}

######################################################################

set MonitorStatusTable {
	{ dial Dial \#00a7ff \#002268 }
	{ chat Chat \#49dc00 \#0a4c00 }
	{ auth Auth \#eb9c00 \#513300 }
	{ network Network \#e10000 \#510000 }
}

proc Monitor_Status { frame } {
	global MonitorStatus MonitorStatusTable UpdatePacketEvent

	frame $frame

	foreach t $MonitorStatusTable {
		set name [lindex $t 0]
		set text [lindex $t 1]
		set bg [lindex $t 3]
		label $frame.$name -relief raised -borderwidth 2 -width 7 \
			-text $text -background $bg
		pack $frame.$name -side left -padx 0
	}
	PPxP_RegisterStatusEvent [list Monitor_StatusUpdate $frame]
}

proc Monitor_SetStatusLabel { label name number } {
	global MonitorStatus MonitorStatusTable

	set t [lindex $MonitorStatusTable $number]
	if { $MonitorStatus($name) } {
		$label configure -background [lindex $t 2]
	} else {
		$label configure -background [lindex $t 3]
	}
}

proc Monitor_StatusUpdate { f } {
	global PPxPStatus MonitorStatusTable

	if { $PPxPStatus > 4 } {
		foreach t $MonitorStatusTable {
			set label [lindex $t 0]
			$f.$label configure -background \#ebd000
		}
		return
	}
	for { set i 0 } { $i < $PPxPStatus } { incr i } {
		set t [lindex $MonitorStatusTable $i]
		set label [lindex $t 0]
		$f.$label configure -background [lindex $t 2]
	}
	for {} { $i < 4 } { incr i } {
		set t [lindex $MonitorStatusTable $i]
		set label [lindex $t 0]
		$f.$label configure -background [lindex $t 3]
	}
}

######################################################################

proc Monitor_Name { w } {
	global PPxPEnv

	label $w -width 12 -relief sunken -justify center \
		-textvariable PPxPEnv(NAME)
}

######################################################################

proc Monitor_IpAddress { frame } {
	global PPxPEnv

	frame $frame
	label $frame.localLabel -text "Local:"
	label $frame.remoteLabel -text "Remote:"
#	IpEntry $frame.netmask -variable PPxPEnv(IP.NETMASK) -netmaskon 0
	IpEntry $frame.localAddress -variable PPxPEnv(IP.LOCAL) \
		-state disabled
	IpEntry $frame.remoteAddress -variable PPxPEnv(IP.REMOTE) \
		-state disabled
#	IpEntry $frame.netmask -variable PPxPEnv(IP.NETMASK) -netmaskon 0

	grid $frame.localLabel    -column 0 -row 0 -sticky e
	grid $frame.remoteLabel   -column 0 -row 1 -sticky e
	grid $frame.localAddress  -column 1 -row 0 -sticky w
	grid $frame.remoteAddress -column 1 -row 1 -sticky w
#	grid $frame.netmaskLabel  -column 0 -row 2 -sticky w
#	grid $frame.netmask       -column 1 -row 2 -sticky w
}

######################################################################

proc Monitor_Buttons { frame main ppxp } {
	frame $frame

	set width [string length "Disconnect"]
	button $frame.connect -text "Connect" -width $width \
		-command [list $ppxp connect]
	button $frame.bye -text "Bye" \
		-command [list Monitor_Bye $ppxp $main]
	button $frame.quit -text "Quit" \
		-command [list Monitor_Quit $ppxp $main]

	pack $frame.connect $frame.bye $frame.quit -side left

	PPxP_RegisterStatusEvent \
		[list Monitor_SetConnectButton $frame.connect $ppxp]
}

proc Monitor_SetConnectButton { button ppxp } {
	global PPxPStatusString
	if { $PPxPStatusString != "dead" } {
		$button configure -text "Disconnect" \
			-command [list $ppxp disconnect]
	} else {
		$button configure -text "Connect" \
			-command [list $ppxp connect]
	}
}

proc Monitor_Bye { ppxp main } {
	$ppxp bye
	if { $main == {} } {
		destroy .
	} else {
		destroy $main
	}
}

proc Monitor_Quit { ppxp main } {
	$ppxp quit
	if { $main == {} } {
		destroy .
	} else {
		destroy $main
	}
}

######################################################################

proc Monitor_LoadConf { ppxp main } {
	set file [PPxP_SelectConf $main.select conf \
			  "Select configuration script"]
	if { $file != "" } {
		$ppxp source $file
		$ppxp env
	}
}

proc Monitor_MenuBar { mbar main ppxp } {
	frame $mbar -relief raised -borderwidth 2

	menubutton $mbar.file -text "File" -menu $mbar.file.menu -under 0
	menubutton $mbar.execute -text "Execute" -menu $mbar.execute.menu \
		-under 1
#	menubutton $mbar.setup -text "Setup" -menu $mbar.setup.menu -under 0
	menubutton $mbar.help -text "Help" -menu $mbar.help.menu -under 0

	menu $mbar.file.menu -tearoff 0
	$mbar.file.menu add command \
		-label "Load configuration..." -underline 0 \
		-command [list Monitor_LoadConf $ppxp $main]
	$mbar.file.menu add separator
	$mbar.file.menu add command \
		-label "Close Console" -underline 0 \
		-command [list Monitor_Bye $ppxp $main]
	$mbar.file.menu add command \
		-label "Quit Console and Daemon" -underline 0 \
		-command [list Monitor_Quit $ppxp $main]

	menu $mbar.execute.menu -tearoff 0
	$mbar.execute.menu add command \
		-label "Connect" -underline 0 \
		-command [list $ppxp connect]
	$mbar.execute.menu add command \
		-label "Disconnect" -underline 0 \
		-command [list $ppxp disconnect]
	$mbar.execute.menu add separator

	global PPxPQDialLoad PPxPSetupLoad
	if $PPxPQDialLoad {
		$mbar.execute.menu add command \
			-label "Quick DialUp" -underline 0 \
			-command [list Monitor_QDialStart $ppxp $main.qdial]
	}
	if $PPxPSetupLoad {
		$mbar.execute.menu add command \
			-label "Setup" -underline 0 \
			-command [list Monitor_SetupStart $ppxp $main.setup]
	}
#	menu $mbar.setup.menu -tearoff 0
	menu $mbar.help.menu -tearoff 0

	pack $mbar.file $mbar.execute -side left
#	pack $mbar.help -side right
}

proc Monitor_QDialStart { ppxp w } {
	if { [winfo exists $w] } {
		raise $w
	} else {
		toplevel $w
		wm title $w "PPxP Quick Dialup"
		QDial_Start $ppxp $w 0
	}
}

proc Monitor_SetupStart { ppxp w } {
	if { [winfo exists $w] } {
		raise $w
	} else {
		toplevel $w
		wm title $w "PPxP Variable Setup"
		Env_Start $ppxp $w 0
	}
}

######################################################################

proc Monitor_Start { ppxp w flag } {
	global Palette MonitorMain
	option add *Label.background $Palette(background)

	if { $flag } {
		set MonitorMain 1
	} else {
		set MonitorMain 0
	}
	frame $w.f1
	frame $w.f2
	frame $w.f3
	frame $w.f4

	Logo24 $w.logo $Palette(logoForeground) $Palette(logoAccentColor) \
		$Palette(logoShadowColor) 1 1
	Monitor_Status    $w.status
	Monitor_Auto      $w.auto   $ppxp
	Monitor_Indicator $w.ind    3
	Monitor_Time      $w.time   $ppxp
	Monitor_IpAddress $w.ipAddr
	Monitor_Name      $w.name
#	Monitor_Buttons   $w.buttons $w $ppxp
	Monitor_MenuBar   $w.menubar $w $ppxp

	button $w.connect -text "Connect" -pady 0
	button $w.disconn -text "Disconnect" -pady 0

	pack $w.logo	-in $w.f1 -side left
	pack $w.auto	-in $w.f1 -side right
	pack $w.status	-in $w.f1 -side left
	pack $w.ind	-in $w.f2 -side left
	pack $w.ipAddr	-in $w.f3 -side left
	pack $w.time	-in $w.f3 -side right
#	pack $w.buttons	-in $w.f4 -side right

	pack $w.menubar -side top -fill x -expand yes
	pack $w.f1 $w.f2 $w.f3 \
		-side top -fill x -expand yes -padx 4 -pady 2
}
