#!/usr/bin/wish -f

package require PPxP

######################################################################

set PPxPSendNSizeOld 0
set PPxPRecvNSizeOld 0
set PPxPInfo(send.nrate) 0
set PPxPInfo(recv.nrate) 0

proc PPxP_RecvHandler { ppxp } {
	global PPxPEnv PPxPInfo PPxPUpdateEvent
	global PPxPSendNSizeOld PPxPRecvNSizeOld
	global PPxP_XCIO_S_OUT PPxP_XCIO_ENV_SET PPxP_XCIO_UP_ENVS \
		PPxP_XCIO_UP_INFO PPxP_XCIO_LISTUP

	if { [catch { set t [$ppxp read s] }] } {
#		close [$ppxp filehandle]
		exit 1
	}
	switch $t \
		$PPxP_XCIO_S_OUT {
			PPxP_SOut $s
		} \
		$PPxP_XCIO_ENV_SET {
			set PPxPEnv([lindex $s 0]) [lindex $s 1]
	} \
		$PPxP_XCIO_UP_ENVS {
			set PPxPEnv([lindex $s 0]) [lindex $s 1]
		} \
		$PPxP_XCIO_LISTUP {
			if { ! [regexp {^CMD} $s] } {
				$ppxp env [lindex $s 1]
			}
		} \
		$PPxP_XCIO_UP_INFO {
			foreach i $s {
				set e [lindex $i 0]
				set v [lindex $i 1]
				if { ![info exists PPxPInfo($e)] ||
				     $PPxPInfo($e) != $v } {
					set PPxPInfo($e) $v
				}
			}
			set PPxPInfo(send.nrate) \
				[expr $PPxPInfo(send.nsize)-$PPxPSendNSizeOld]
			set PPxPInfo(recv.nrate) \
				[expr $PPxPInfo(recv.nsize)-$PPxPRecvNSizeOld]
			set PPxPSendNSizeOld $PPxPInfo(send.nsize)
			set PPxPRecvNSizeOld $PPxPInfo(recv.nsize)

			set PPxPUpdateEvent 1
		}
}

set PPxPStatusList {
	dead dial chat auth network term
}
set PPxPStatus -1
set PPxPStatusString ""
set PPxPStatusMax [llength $PPxPStatusList]
set PPxPStatusPrev ""

proc PPxP_StatusUpdate {} {
	global PPxPInfo
	global PPxPStatus PPxPStatusString PPxPStatusMax PPxPStatusList
	global PPxPStatusPrev

	if { $PPxPInfo(phase) == "dead" && $PPxPInfo(linestat) == "" } {
		set status 0
	} elseif { $PPxPInfo(phase) == "dead" &&
		   [regexp {dial} $PPxPInfo(linestat)] } {
		set status 1
	} elseif { $PPxPInfo(phase) == "dead" &&
		   [regexp {tty} $PPxPInfo(linestat)] } {
		set status 2
	} elseif { $PPxPInfo(phase) == "authenticate" } {
		set status 3
	} elseif { $PPxPInfo(phase) == "network" &&
		   [regexp {nlink} $PPxPInfo(linestat)] } {
		set status 4
	} elseif { $PPxPInfo(phase) == "terminate" } {
		set status 5
	} else {
		set status -1
	}
	if { $status >= 0 && $status != $PPxPStatus } {
		set PPxPStatusPrev $PPxPStatus
		set PPxPStatusString [lindex $PPxPStatusList $status]
		set PPxPStatus $status
	}
}

set PPxPSOutHandlerList {}
proc PPxP_RegisterSOut { cmd } {
	global PPxPSOutHandlerList
	lappend PPxPSOutHandlerList $cmd
	return [expr [llength $PPxPSOutHandlerList] - 1]
}

proc PPxP_UnRegisterSOut { id } {
	global PPxPSOutHandlerList
	set PPxPSOutHandlerList [lreplace $PPxPSOutHandlerList $id $id {}]
}

proc PPxP_SOut { s } {
	global PPxPSOutHandlerList

	foreach cmd $PPxPSOutHandlerList {
		if { $cmd != "" } {
			regsub -all {%s} $cmd \"$s\" cmd
			eval $cmd
		}
	}
}

set PPxPUpdateHandlerId 0
proc PPxP_RegisterUpdateEvent { cmd } {
	global PPxPUpdateEvent PPxPUpdateHandlerId

	set id $PPxPUpdateHandlerId
	proc PPxP_UpdateHandler_$id { name elem op } [list eval $cmd]
	trace variable PPxPUpdateEvent w PPxP_UpdateHandler_$id

	incr PPxPUpdateHandlerId
	return $id
}

proc PPxP_UnRegisterUpdateEvent { id } {
	global PPxPUpdateEvent
	trace vdelete PPxPUpdateEvent w PPxP_UpdateHandler_$id
	rename PPxP_UpdateHandler_$id {}
}

set PPxPInfoHandlerId 0
proc PPxP_RegisterInfoEvent { elem cmd } {
	global PPxPInfo PPxPInfoHandlerId

	set id $PPxPInfoHandlerId
	proc PPxP_InfoHandler_$id { name elem op } [list eval $cmd]
	trace variable PPxPInfo($elem) w PPxP_InfoHandler_$id

	incr PPxPInfoHandlerId
	return $id
}

proc PPxP_UnRegisterInfoEvent { elem id } {
	global PPxPInfo
	trace vdelete PPxPInfo($elem) w PPxP_InfoHandler_$id
	rename PPxP_InfoHandler_$id {}
}

set PPxPEnvHandlerId 0
proc PPxP_RegisterEnvEvent { elem cmd } {
	global PPxPEnv PPxPEnvHandlerId

	set id $PPxPEnvHandlerId
	proc PPxP_EnvHandler_$id { name elem op } [list eval $cmd]
	trace variable PPxPEnv($elem) w PPxP_EnvHandler_$id

	incr PPxPEnvHandlerId
	return $id
}

proc PPxP_UnRegisterEnvEvent { elem id } {
	global PPxPEnv
	trace vdelete PPxPEnv($elem) w PPxP_EnvHandler_$id
	rename PPxP_EnvHandler_$id {}
}

set PPxPStatusHandlerId 0
proc PPxP_RegisterStatusEvent { cmd } {
	global PPxPStatus PPxPStatusHandlerId

	set id $PPxPStatusHandlerId
	proc PPxP_StatusHandler_$id { name elem op } \
		[list PPxP_StatusHandler $cmd]
	trace variable PPxPStatus w PPxP_StatusHandler_$id

	incr PPxPStatusHandlerId
	return $id
}

proc PPxP_UnRegisterStatusEvent { id } {
	global PPxPStatus
	trace vdelete PPxPStatus w PPxP_StatusHandler_$id
	rename PPxP_StatusHandler_$id {}
}

proc PPxP_StatusHandler { cmd } {
	global PPxPStatus PPxPStatusPrev
	regsub -all {%C} $cmd $PPxPStatus cmd
	regsub -all {%P} $cmd $PPxPStatusPrev cmd
	eval $cmd
}

######################################################################

proc PPxP_GetAllEnv { ppxp } {
	global PPxP_XCIO_LAST PPxPEnv

	set id [$ppxp listup]
	set vars {}
	while { 1 } {
		if { [catch { set t [$ppxp read s $id] }] } {
			break
		}
		if { ($t & ~$PPxP_XCIO_LAST) > 0 } {
			if { ![regexp {^CMD} $s] } {
				lappend vars [lindex $s 1]
			}
		}
		if { $t & $PPxP_XCIO_LAST } {
			break
		}
	}
	foreach v $vars {
		set id [$ppxp env $v]
		while { 1 } {
			if [catch { set t [$ppxp read s $id] }] {
				break
			}
			if { ($t & ~$PPxP_XCIO_LAST) > 0 } {
				set PPxPEnv([lindex $s 0]) [lindex $s 1]
			}
			if { $t == 0 } {
				# update
			}
			if { $t & $PPxP_XCIO_LAST } {
				if { ![info exists PPxPEnv($v)] } {
					set PPxPEnv($v) {}
				}
				break
			}
		}
	}
}

######################################################################

proc PPxP_Start { av } {
	set ppxp [eval PPxP_Setup $av]

	PPxP_GetAllEnv $ppxp
	$ppxp update
	$ppxp autoupdate

	PPxP_RegisterUpdateEvent PPxP_StatusUpdate
	PPxP_RegisterInfoEvent netstat [list $ppxp env IP]

	fileevent [$ppxp filehandle] readable [list PPxP_RecvHandler $ppxp]

	return $ppxp
}
