######################################################################

switch -glob [exec uname -s] {
	Linux {
		set QDialDeviceList \
			[glob -nocomplain \
				 "/dev/modem" "/dev/ttyS*" "/dev/cua*"]
	}
	FreeBSD {
		set QDialDeviceList \
			[glob -nocomplain "/dev/ttyd*" "/dev/cuaa*"]
	}
	* {
		set QDialDeviceList [lsort [glob -nocomplain "/dev/*"]]
	}
}

######################################################################

set QDialCurrentState ""
set QDialItemEntryWidth 20
set QDialConnectTime "00:00:00"
set QDialCurrentHelp ""

if { ![info exists env(LANG)] } {
	set env(LANG) "C"
}

if { [info commands kanji] == "kanji" && [regexp {^ja} $env(LANG)] } {
	source [file join $TkPPxPDir qdial_ja.tcl]
	set QDialHelpWrap char
} else {
	source [file join $TkPPxPDir qdial_C.tcl]
	set QDialHelpWrap word
}

if [file exists [file join $TkPPxPDir qdial_start.gif]] {
	image create photo QDialStartImage \
		-file [file join $TkPPxPDir qdial_start.gif]
	set QDialStartImage QDialStartImage
} else {
	set QDialStartImage {}
}

######################################################################

proc QDial_Logo { w } {
	global Palette
	Logo24 $w $Palette(logoForeground) \
		$Palette(logoAccentColor) \
		$Palette(logoShadowColor) 1 1
}

proc QDial_TitleFrame { w } {
	global QDialTitle Palette

	frame $w
	QDial_Logo $w.logo
	label $w.desc -text $QDialTitle(desc) \
		-background $Palette(background)
	pack $w.logo -side left
	pack $w.desc -fill x -expand yes -side left
}

######################################################################
proc QDial_ModemList {} {
	global PPxP_SysPath

#	set path [file join $PPxP_SysPath modem]
	set flist [glob -nocomplain [file join $PPxP_SysPath modem *]]
	set mlist ""

	foreach fname $flist {
		set f [open $fname]
		set name [file tail $fname]
		set desc ""
		while { [gets $f l] >= 0 } {
			if [regexp -nocase "name" [lindex $l 0]] {
				set desc [lindex $l 1]
				break
			}
		}
		close $f
		set m [format "%-12s(%s)" $name $desc]
		set mlist [lappend mlist $m]
	}
	return [lsort $mlist]
}

proc QDial_ShowHelp { i c } {
	global QDialHelp QDialButtonHelp QDialCurrentHelp QDialPrevHelp

	set QDialPrevHelp $QDialCurrentHelp
	if { $c == "button" } {
		set QDialCurrentHelp $QDialButtonHelp($i)
	} else {
		set QDialCurrentHelp $QDialHelp($i)
	}
}

proc QDial_PrevHelp {} {
	global QDialCurrentHelp QDialPrevHelp
	set QDialCurrentHelp $QDialPrevHelp
}

proc QDial_ItemEntry { e i } {
	global QDialItemEntryWidth PPxPEnv
	entry $e -width $QDialItemEntryWidth \
		-textvariable PPxPEnv($i)
	SetEntryState $e normal
	bind $e <FocusIn> [list QDial_ShowHelp $i {}]
}

proc QDial_MenuEntry { frame item itemlist } {
	global QDialItemEntryWidth PPxPEnv

	MenuEntry $frame -width [expr $QDialItemEntryWidth - 3] \
		-variable PPxPEnv($item) -items $itemlist
	bind [$frame subwidget entry] <FocusIn> \
		[list QDial_ShowHelp $item {}]
	bind [$frame subwidget button] <FocusIn> \
		[list QDial_ShowHelp $item {}]
}

proc QDial_UpDownEntry { w item } {
	global QDialItemEntryWidth

	UpDownEntry $w -width [expr $QDialItemEntryWidth - 3] \
		-variable PPxPEnv($item) -minimum 0
	bind [$w subwidget entry] <FocusIn> [list QDial_ShowHelp $item {}]
}

proc QDial_PasswdFake { w k } {
	if { [string length $k] > 0 && [string length [$w get]] == 0 } {
		$w insert end "\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034\034"
	}
}

proc QDial_PasswdStrip { p } {
	regsub -all "\034" $p {} passwd
	return $passwd
}

proc QDial_PasswdEntry { w item } {
	QDial_ItemEntry $w $item
	$w configure -show *
	bind $w <Key> { QDial_PasswdFake %W %A }
	foreach i { Control Meta Alt } {
		bind $w <$i-Key> { return }
	}
	foreach i { Return Tab Delete } {
		bind $w <Key-$i> { return }
	}
}

proc QDial_InputFrame { w } {
	global Palette
	global PPxPEnv
	global QDialEnv
	global QDialLabel
	global QDialCurrentHelp
	global QDialDeviceList
	global QDialStartImage
	global QDialMessage

	frame $w

	label $w.devicelabel -text $QDialLabel(LINE) \
		-background $Palette(labelBackground)
	label $w.diallabel   -text $QDialLabel(DIAL.TYPE) \
		-background $Palette(labelBackground)
	label $w.numberlabel -text $QDialLabel(DIAL.LIST) \
		-background $Palette(labelBackground)
	label $w.namelabel   -text $QDialLabel(user) \
		-background $Palette(labelBackground)
	label $w.passwd1label -text $QDialLabel(passwd1) \
		-background $Palette(labelBackground)
	label $w.passwd2label -text $QDialLabel(passwd2) \
		-background $Palette(labelBackground)

	label $w.modemlabel  -text $QDialLabel(SERIAL.MODEM) \
		-background $Palette(labelBackground)
	label $w.authlabel   -text $QDialLabel(AUTH.PROTO) \
		-background $Palette(labelBackground)
	label $w.modelabel   -text $QDialLabel(MODE) \
		-background $Palette(labelBackground)
	label $w.idlelabel   -text $QDialLabel(IDLE.INIT) \
		-background $Palette(labelBackground)
	label $w.vjlabel     -text $QDialLabel(IP.VJ) \
		-background $Palette(labelBackground)
	label $w.autolabel   -text $QDialLabel(auto) \
		-background $Palette(labelBackground)

	QDial_MenuEntry $w.deviceentry LINE $QDialDeviceList
	QDial_MenuEntry $w.dialentry DIAL.TYPE \
		{ Tone Pulse Data Isdn Piafs none }
	QDial_ItemEntry $w.numberentry DIAL.LIST
	QDial_ItemEntry $w.nameentry user
	QDial_PasswdEntry $w.passwd1entry passwd1
	QDial_PasswdEntry $w.passwd2entry passwd2
	QDial_MenuEntry $w.modementry SERIAL.MODEM [QDial_ModemList]
	QDial_MenuEntry $w.authentry AUTH.PROTO \
		{ {PAP CHAP} {CHAP PAP} PAP CHAP none }
	QDial_MenuEntry $w.modeentry MODE {active passive kick}
	QDial_UpDownEntry $w.idleentry IDLE.INIT

	frame $w.vjentry
	radiobutton $w.vjentry.yes -text yes -value yes \
		-variable PPxPEnv(IP.VJ)
	radiobutton $w.vjentry.no -text no -value no \
		-variable PPxPEnv(IP.VJ)
	pack $w.vjentry.yes $w.vjentry.no -side left

	frame $w.autoentry
	radiobutton $w.autoentry.on -text on -value on \
		-variable QDialEnv(auto)
	radiobutton $w.autoentry.off -text off -value off \
		-variable QDialEnv(auto)
	pack $w.autoentry.on $w.autoentry.off -side left
	set id [PPxP_RegisterInfoEvent mode QDial_SetAutoMode]
	bind $w.autoentry <Destroy> \
		[list PPxP_UnRegisterInfoEvent mode $id]

	$w.dialentry configure -state buttononly
	$w.authentry configure -state buttononly
	$w.modeentry configure -state buttononly

#	label $w.help -width 42 -relief sunken \
#		-background $Palette(background)
	global QDialHelpWrap
	text $w.help -width 42 -height 10 -relief sunken -state disabled \
		-wrap $QDialHelpWrap -yscrollcommand "$w.helpscroll set"
	scrollbar $w.helpscroll -orient vertical -command "$w.help yview"
		
	proc QDial_InsertHelp { w } {
		global QDialCurrentHelp
		$w configure -state normal
		$w delete 0.0 end
		$w insert end $QDialCurrentHelp
		$w configure -state disabled
	}
	proc QDial_HelpTracer { name elem op } \
		[list QDial_InsertHelp $w.help]
	trace variable QDialCurrentHelp w QDial_HelpTracer

	if { [string length $QDialStartImage] > 0 } {
		label $w.help.start -image $QDialStartImage
		$w.help window create insert -window $w.help.start \
			-align baseline
	}

	frame $w.more
	button $w.more.button -image ImageDown
	label $w.more.label -text $QDialMessage(more_entry) \
		-background $Palette(background)
	pack $w.more.button $w.more.label -side left

	grid $w.devicelabel -column 0 -row 0 -sticky ew
	grid $w.deviceentry -column 1 -row 0 -sticky ew
	grid $w.diallabel   -column 0 -row 1 -sticky ew
	grid $w.dialentry   -column 1 -row 1 -sticky ew
	grid $w.numberlabel -column 0 -row 2 -sticky ew
	grid $w.numberentry -column 1 -row 2 -sticky ew
	grid $w.namelabel   -column 0 -row 3 -sticky ew
	grid $w.nameentry   -column 1 -row 3 -sticky ew
	grid $w.passwd1label -column 0 -row 4 -sticky ew
	grid $w.passwd1entry -column 1 -row 4 -sticky ew
	grid $w.passwd2label -column 0 -row 5 -sticky ew
	grid $w.passwd2entry -column 1 -row 5 -sticky ew

	grid $w.more -column 0 -row 6 -sticky w -columnspan 2

	grid $w.modemlabel  -column 0 -row 7 -sticky ew
	grid $w.modementry  -column 1 -row 7 -sticky ew
	grid $w.idlelabel   -column 2 -row 7 -sticky ew
	grid $w.idleentry   -column 3 -row 7 -sticky ew
	grid $w.authlabel   -column 0 -row 8 -sticky ew
	grid $w.authentry   -column 1 -row 8 -sticky ew
	grid $w.modelabel   -column 2 -row 8 -sticky ew
	grid $w.modeentry   -column 3 -row 8 -sticky ew
	grid $w.vjlabel     -column 0 -row 9 -sticky ew
	grid $w.vjentry     -column 1 -row 9 -sticky ew
#	grid $w.autolabel   -column 2 -row 9 -sticky ew
#	grid $w.autoentry   -column 3 -row 9 -sticky ew

	grid $w.help -column 2 -row 0 -sticky ns -columnspan 2 -rowspan 7
	grid $w.helpscroll -column 4 -row 0 -sticky ns -columnspan 1 -rowspan 7

	QDial_HideMore $w $w.more.button
}

proc QDial_SetAutoMode {} {
	global PPxPInfo QDialEnv
	if { $PPxPInfo(mode) == "auto" } {
		set QDialEnv(auto) on
	} else {
		set QDialEnv(auto) off
	}
}

proc QDial_HideMore { w b } {
	global QDialCurrentHelp

	foreach i { modemlabel modementry authlabel authentry
		modelabel modeentry idlelabel idleentry
		vjlabel vjentry } {
		grid remove $w.$i
	}
#	grid $w.help -rowspan 7
	$b configure -command [list QDial_ShowMore $w $b]
}

proc QDial_ShowMore { w b } {
	foreach i { modemlabel modementry authlabel authentry
		modelabel modeentry idlelabel idleentry
		vjlabel vjentry } {
		grid $w.$i
	}
#	grid $w.help -rowspan 12
	$b configure -command [list QDial_HideMore $w $b]
}

######################################################################

proc QDial_ComFrame { w main ppxp } {
	global QDialButtonText QDialMain
	global Palette

	frame $w
	frame $w.buttons
	frame $w.state

	if { $QDialMain } {
		set buttons { dial apply save close quit }
	} else {
		set buttons { dial apply save close }
	}
	foreach b $buttons {
		button $w.buttons.$b -text $QDialButtonText($b)
		pack $w.buttons.$b -side left

		bind $w.buttons.$b <FocusIn> \
			[list QDial_ShowHelp $b button]
		bind $w.buttons.$b <Enter> \
			[list QDial_ShowHelp $b button]
		bind $w.buttons.$b <Leave> { QDial_PrevHelp }
	}
	label $w.state.time -background $Palette(background) -text "00:00:00"
	label $w.state.phase -background $Palette(background)
	pack $w.state.time -side right
	pack $w.state.phase -side right -fill x -expand yes

	pack $w.buttons -side left
	pack $w.state -side left -fill x -expand yes

	$w.buttons.apply configure -command [list QDial_Apply $ppxp]
	$w.buttons.save configure -command [list QDial_Save $ppxp]
	$w.buttons.close configure -command [list QDial_Close $ppxp $main]
	if { $QDialMain } {
		$w.buttons.quit configure -command \
			[list QDial_Quit $ppxp $main]
	}
	set id [PPxP_RegisterInfoEvent connect \
			[list QDial_ConnectTimeUpdate $w.state.time]]
	bind $w.state.time <Destroy> \
		[list PPxP_UnRegisterInfoEvent connect $id]

	set id [PPxP_RegisterStatusEvent \
			[list QDial_StatusLabelUpdate $w.state.phase]]
	bind $w.state.phase <Destroy> [list PPxP_UnRegisterStatusEvent $id]

	set id [PPxP_RegisterStatusEvent \
			[list QDial_DialButtonUpdate $w.buttons.dial $ppxp]]
	bind $w.buttons.dial <Destroy> [list PPxP_UnRegisterStatusEvent $id]

	$w.buttons.dial configure \
		-command [list QDial_Connect $ppxp] \
		-text $QDialButtonText(dial)
	bind $w.buttons.dial <FocusIn> {
		QDial_ShowHelp dial button
	}
	bind $w.buttons.dial <Enter> {
		QDial_ShowHelp dial button
	}
}

proc QDial_ConnectTimeUpdate { label } {
	global PPxPInfo

	set t $PPxPInfo(connect)
	set h [expr $t/3600]
	set m [expr ($t%3600)/60]
	set s [expr $t%60]
	$label configure -text [format "%02d:%02d:%02d" $h $m $s]
}

proc QDial_StatusLabelUpdate { label } {
	global PPxPStatusString QDialState
	$label configure -text $QDialState($PPxPStatusString)
}

proc QDial_DialButtonUpdate { button ppxp } {
	global PPxPStatusString QDialButtonText

	if { $PPxPStatusString == "dead" } {
		$button configure \
			-command [list QDial_Connect $ppxp] \
			-text $QDialButtonText(dial)
		bind $button <FocusIn> {
			QDial_ShowHelp dial button
		}
		bind $button <Enter> {
			QDial_ShowHelp dial button
		}
	} else {
		$button configure \
			-command [list QDial_DisConnect $ppxp] \
			-text $QDialButtonText(dis)
		bind $button <FocusIn> {
			QDial_ShowHelp dis button
		}
		bind $button <Enter> {
			QDial_ShowHelp dis button
		}
	}
}

######################################################################

proc QDial_DialEnable {} {
	global PPxPEnv QDialMessage

	if { $PPxPEnv(AUTH.PROTO) == "none" &&
	     [string length $PPxPEnv(LINE)]         > 0 &&
	     [string length $PPxPEnv(SERIAL.MODEM)] > 0 &&
	     [string length $PPxPEnv(DIAL.LIST)]    > 0 &&
	     [string length $PPxPEnv(DIAL.TYPE)]    > 0 } {
		return 1
	} elseif { [string length $PPxPEnv(LINE)]  > 0 &&
		   [string length $PPxPEnv(SERIAL.MODEM)] > 0 &&
		   [string length $PPxPEnv(DIAL.TYPE)]    > 0 &&
		   [string length $PPxPEnv(DIAL.LIST)]    > 0 &&
		   [string length $PPxPEnv(AUTH.PROTO)]   > 0 &&
		   [string length $PPxPEnv(passwd1)]      > 0 && 
		   [string length $PPxPEnv(passwd2)]      > 0 } {
		set passwd1 [QDial_PasswdStrip $PPxPEnv(passwd1)]
		set passwd2 [QDial_PasswdStrip $PPxPEnv(passwd2)]
		if { $passwd1 != $passwd2 } {
			tk_messageBox -type ok \
				-message $QDialMessage(inconsist_passwd)
			return 0
		} else {
			return 1
		}
	} else {
		tk_messageBox -type ok -message $QDialMessage(empty_entries)
		return 0
	}
}

proc QDial_Apply { ppxp } {
	global PPxPEnv

	$ppxp set CHAT none
	$ppxp source qdial

#	puts "$PPxPEnv(device) $PPxPEnv(modem) $PPxPEnv(dial) \
# $PPxPEnv(number) $PPxPEnv(auth)"
	if { ![QDial_DialEnable] } {
		return
	}
	if { [string length $PPxPEnv(LINE)] > 0 } {
		$ppxp set LINE	$PPxPEnv(LINE)
	}
	if { [string length $PPxPEnv(SERIAL.MODEM)] > 0 } {
		$ppxp set SERIAL.MODEM [lindex $PPxPEnv(SERIAL.MODEM) 0]
	}
	if { [string length $PPxPEnv(DIAL.TYPE)] > 0 } {
		$ppxp set DIAL.TYPE $PPxPEnv(DIAL.TYPE)
	}
	if { [string length $PPxPEnv(DIAL.LIST)] > 0 } {
		$ppxp set DIAL.LIST $PPxPEnv(DIAL.LIST)
	}
	if { [string length $PPxPEnv(AUTH.PROTO)] > 0 } {
		eval $ppxp set AUTH.PROTO $PPxPEnv(AUTH.PROTO)
	}
	if { [string length $PPxPEnv(MODE)] > 0 } {
		$ppxp set MODE $PPxPEnv(MODE)
	}
	if { [string length $PPxPEnv(IDLE.INIT)] > 0 } {
		$ppxp set IDLE.INIT $PPxPEnv(IDLE.INIT)
	}
	if { [string length $PPxPEnv(passwd1)] > 0 &&
	     [string length $PPxPEnv(passwd2)] > 0 &&
	     [string length $PPxPEnv(user)] > 0 } {
		set passwd1 [QDial_PasswdStrip $PPxPEnv(passwd1)]
		set passwd2 [QDial_PasswdStrip $PPxPEnv(passwd2)]
		if { $passwd1 == $passwd2 } {
			$ppxp passwd {} $PPxPEnv(user) $passwd1
		}
	}
	if { [string length $PPxPEnv(IP.VJ)] > 0 } {
		$ppxp set IP.VJ $PPxPEnv(IP.VJ)
	}
}

proc QDial_Connect { ppxp } {
	global QDialMessage PPxPEnv

	if { ![QDial_DialEnable] } {
		return
	}
	QDial_Apply $ppxp
	$ppxp connect
}

proc QDial_DisConnect { ppxp } {
	$ppxp disconnect
}

proc QDial_Save { ppxp } {
	global PPxPEnv
	global PPxP_UsrPath PPxP_SysPath

	if { ![QDial_DialEnable] } {
		return
	}
	if [info exists PPxP_UsrPath] {
		set path $PPxP_UsrPath
	} else {
		set path $PPxP_SysPath
	}
	set fname [tk_getSaveFile -initialdir [join "$path conf" "/"]]
	if { [string length $fname] <= 0 } {
		return;
	}
	regexp {[^/]+$} $fname entry
	if { [string length $PPxPEnv(passwd1)] > 0 &&
	     [string length $PPxPEnv(passwd2)] > 0 &&
	     [string length $PPxPEnv(user)] > 0 } {
		set p1 [QDial_PasswdStrip $PPxPEnv(passwd1)]
		set p2 [QDial_PasswdStrip $PPxPEnv(passwd2)]
		if { $p1 == $p2 } {
			$ppxp passwd \
				$entry $PPxPEnv(user) $p1
		}
	}
	set f [open $fname w]
	puts $f "source qdial"
	puts $f "set AUTH.PASSWD $entry"
	if { [string length $PPxPEnv(MODE)] > 0 } {
		puts $f "set MODE $PPxPEnv(MODE)"
	}
	if { [string length $PPxPEnv(LINE)] > 0 } {
		puts $f "set LINE $PPxPEnv(LINE)"
	}
	if { [string length $PPxPEnv(SERIAL.MODEM)] > 0 } {
		puts $f "set SERIAL.MODEM $PPxPEnv(SERIAL.MODEM)"
	}
	if { [string length $PPxPEnv(DIAL.TYPE)] > 0 } {
		puts $f "set DIAL.TYPE $PPxPEnv(DIAL.TYPE)"
	}
	if { [string length $PPxPEnv(DIAL.LIST)] > 0 } {
		puts $f "set DIAL.LIST $PPxPEnv(DIAL.LIST)"
	}
	if { [string length $PPxPEnv(AUTH.PROTO)] > 0 } {
		puts $f "set AUTH.PROTO $PPxPEnv(AUTH.PROTO)"
	}
# 	if { $PPxPEnv(AUTH.PROTO) == "none" } {
# 		puts $f "set MODE passive"
# 	}
	if { [string length $PPxPEnv(IDLE.INIT)] > 0 } {
		puts $f "set IDLE.INIT $PPxPEnv(IDLE.INIT)"
	}
	if { [string length $PPxPEnv(IP.VJ)] > 0 } {
		puts $f "set IP.VJ $PPxPEnv(IP.VJ)"
	}
	close $f
}

proc QDial_Quit { ppxp frame } {
	global QDialMain

	if { $QDialMain } {
		$ppxp quit
	}
	if { $frame == {} } {
		destroy .
	} else {
		destroy $frame
	}
}

proc QDial_Close { ppxp frame } {
	global QDialMain

	if { $QDialMain } {
		$ppxp bye
	}
	if { $frame == {} } {
		destroy .
	} else {
		destroy $frame
	}
}

######################################################################

proc QDial_Start { ppxp main flag } {
	global PPxPEnv QDialMain

	if { $flag } {
		set QDialMain 1
	} else {
		set QDialMain 0
	}
	QDial_TitleFrame $main.title
	QDial_InputFrame $main.input
	QDial_ComFrame $main.com $main $ppxp
#	QDial_Terminal $main.term $ppxp

	pack $main.title -side top -fill x -expand yes
	pack $main.input -side top -fill both -expand yes
	pack $main.com -side top -fill x -expand yes
#	pack $main.term

	$ppxp source qdial
	$ppxp env LINE
	$ppxp env SERIAL.MODEM
	$ppxp env DIAL.TYPE
	$ppxp env DIAL.LIST
	$ppxp env AUTH.PROTO
	$ppxp env MODE
	$ppxp env IDLE.INIT
	$ppxp env AUTH.PASSWD

	trace variable PPxPEnv(SERIAL.MODEM) w QDial_EnvSerialModemSync
#	trace variable PPxPEnv w QDial_PPxPEnvProc
}

proc QDial_PPxPEnvProc { name elem op } {
	puts "$name:$elem:$op"
}

proc QDial_EnvSerialModemSync { name elem op } {
	global PPxPEnv
	set PPxPEnv(SERIAL.MODEM) [lindex [split $PPxPEnv(SERIAL.MODEM)] 0]
}
