######################################################################

if { $tk_version >= 8.0 } {
	if { [info commands kanji] == "kanji" } {
		font create k14:a14 -compound {a14 k14}
		font create k14:bold \
			-compound {"-*-helvetica-bold-r-normal-*-12-*" k14}
		set medium "k14:a14"
		set bold   "k14:bold"
	} else {
		set medium "a14"
		set bold   "-*-helvetica-bold-r-normal-*-12-*"
	}
	option add *font		$medium
	option add *Label.font		$bold
	option add *Button.font		$bold
	option add *Menu.font		$bold
	option add *Menubutton.font	$bold
	option add *Checkbutton.font	$bold
	option add *Radiobutton.font	$bold
} else {
	set medium	"a14"
	set kmedium	"k14"
	set bold	"-*-helvetica-bold-r-normal-*-12-*"
	set kbold	"k14"
	option add *font		$medium
	option add *kanjiFont		$kmedium
	option add *Label.font		$bold
	option add *Button.font		$bold
	option add *Menu.font		$bold
	option add *Menubutton.font	$bold
	option add *Checkbutton.font	$bold
	option add *Radiobutton.font	$bold
}

bind Entry <Control-Key-u> { %W delete 0 end }

######################################################################

set PaletteList1 {
	{ background		"#d9d9d9" }
	{ foreground		"black"   }
	{ activeBackground	"#ececec" }
	{ activeForeground	"black"   }
	{ disabledForeground	"#a3a3a3" }
	{ insertBackground	"black"   }
	{ hilightBackground	"#d9d9d9" }
	{ hilightColor		"black"   }
	{ troughColor		"gray"    }
	{ selectBackground	"#c3c3c3" }
	{ selectForeground	"black"   }
}
set PaletteList2 {
	{ disabledBackground	"#d9d9d9"   }
	{ writableBackground	"white"     }
	{ readonlyBackground	"white"     }
	{ labelBackground	"gray"      }
	{ labelBackground2	"#cccccc"   }
	{ logoForeground	"SlateBlue" }
	{ logoAccentColor	"OrangeRed" }
	{ logoShadowColor	"gray60"    }
}

foreach l $PaletteList1 {
	set Palette([lindex $l 0]) [lindex $l 1]
}
foreach l $PaletteList2 {
	set Palette([lindex $l 0]) [lindex $l 1]
}

eval tk_setPalette [join $PaletteList1]
option add *Label.background $Palette(background)

######################################################################

set ImageList {
	{ ImageDown	down.xbm	black }
	{ ImageMDown	mini-down.xbm	black }
	{ ImageMUp	mini-up.xbm	black }
	{ ImagePause	circle.xbm	\#e10000}
	{ ImageDRight	double-right.xbm black }
}

foreach l $ImageList {
	set name [lindex $l 0]
	set file [lindex $l 1]
	set fg   [lindex $l 2]
	image create bitmap $name -file [file join $TkPPxPDir $file] \
		-foreground $fg
}

######################################################################

set ListboxMenuHeight 8
set ListboxMenuPrevFocus ""

set ListboxMenuProc(create)	ListboxMenu_Create
set ListboxMenuProc(configure)	ListboxMenu_Config
set ListboxMenuProc(get)	ListboxMenu_Get
set ListboxMenuProc(add)	ListboxMenu_Add
set ListboxMenuProc(post)	ListboxMenu_Post
set ListboxMenuProc(unpost)	ListboxMenu_Unpost
set ListboxMenuProc(activate)	ListboxMenu_Activate

proc ListboxMenu { w args } {
	global ListboxMenuProc

	return [$ListboxMenuProc(create) $w $args]
}

proc ListboxMenu_Proc { w a } {
	global ListboxMenuProc
	set c [shift a]
	return [$ListboxMenuProc($c) $w $a]
}

proc ListboxMenu_Create { w a } {
	global Palette

	frame $w
	rename $w ListboxMenu_$w

	toplevel $w.top
	wm withdraw $w.top
	wm overrideredirect $w.top true
	wm focusmodel $w.top active

	listbox $w.top.items -relief sunken -height 0 -highlightthickness 0 \
		-yscrollcommand "$w.top.yscroll set"
	scrollbar $w.top.yscroll \
		-orient vertical -command "$w.top.items yview"

	ListboxMenu_Config $w $a

	pack $w.top.items -side left -fill both -expand yes
	pack $w.top.yscroll -side left -fill y

	bind $w.top <ButtonRelease-1>  { ListboxMenu_Select %W %x %y }
	bind $w.top.items <Key-Return> { ListboxMenu_Select %W -1 -1 }
	bind $w.top.items <Key-Escape> { ListboxMenu_Cancel %W }

	proc $w { args } [list eval [join [list ListboxMenu_Proc $w {$args}]]]

	return $w
}

proc ListboxMenu_Config { w a } {
	set conf ListboxMenuConfig_$w
	global $conf

	while { [llength $a] > 0 } {
		set o [shift a]

		if { $o == "-width" } {
			set v [shift a]
			$w.top.items configure -width $v
		} elseif { $o == "-height" } {
			set v [shift a]
			$w.top.items configure -height $v
		} elseif { $o == "-scrollbarwidth" } {
			set v [shift a]
			$w.top.yscroll configure -width $v
		} elseif { $o == "-variable" } {
			set v [shift a]
			set ${conf}($o) $v
		}
	}
}

proc ListboxMenu_Post { w a } {
	global ListboxMenuPrevFocus

	set ListboxMenuPrevFocus [focus]

	set p [winfo parent $w]
	set x [winfo rootx $p]
	set y [expr [winfo rooty $p] + [winfo height $p]]
	wm geometry $w.top +$x+$y
	wm deiconify $w.top

	grab set -global $w.top
	raise $w.top $p
	focus $w.top.items
}

proc ListboxMenu_Unpost { w a } {
	global ListboxMenuPrevFocus

	grab release $w.top
	wm withdraw $w.top
	if { [string length $ListboxMenuPrevFocus] > 0 } {
		focus $ListboxMenuPrevFocus
	}
}

proc ListboxMenu_Get { w a } {
	eval $w.top.items get $a
}

proc ListboxMenu_Add { w a } {
	global ListboxMenuHeight
	foreach i $a {
		set h [$w.top.items cget -height]
		if { $h <= $ListboxMenuHeight } {
			incr h
			$w.top.items configure -height $h
		}
		$w.top.items insert end $i
	}
}

proc ListboxMenu_Activate { w a } {
	set i [lindex $a 0]
	$w.top.items activate $i
	$w.top.items selection set $i $i
}

proc ListboxMenu_Select { w x y } {
	set result ""

	if { [regexp {\.top$} $w] } {
		set top [winfo parent $w]
		ListboxMenu_Unpost $top {}
		return $result
	} elseif { [regexp {\.yscroll$} $w] } {
		set top [winfo parent [winfo parent $w]]
		return $result
	} elseif { [regexp {\.items$} $w] } {
		set top [winfo parent [winfo parent $w]]

		if { $x > 0 } {
			tkCancelRepeat
			$w activate @$x,$y
		}
		set c [$w curselection]
		if { $c >= 0 } {
			set result [$w get $c]
		}
		set conf ListboxMenuConfig_$top
		global $conf
		if { [catch { set varname [set ${conf}(-variable)] }] == 0 } {
#			puts "$varname:$result"
			if { $varname != "" && $result != "" } {
				upvar \#0 $varname var
				set var $result
			}
		}
		ListboxMenu_Unpost $top {}
		return $result
	}
}

proc ListboxMenu_Cancel { w } {
	if { [regexp {\.top$} $w] } {
		set top [winfo parent $w]
	} else {
		set top [winfo parent [winfo parent $w]]
	}
	ListboxMenu_Unpost $top {}
	return ""
}

######################################################################

proc SetEntryState { e s } {
	global Palette
	switch $s {
		"normal" {
			$e configure -state normal -takefocus 1 \
				-background $Palette(writableBackground)
		}
		"disabled" {
			$e configure -state disabled -takefocus 0 \
				-background $Palette(disabledBackground)
		}
		"readonly" {
			$e configure -state disabled -takefocus 1 \
				-background $Palette(readonlyBackground)
		}
	}
}

proc SetLabelState { l s } {
	global Palette
	switch $s {
		"normal" {
			$l configure -background $Palette(writableBackground)
		}
		"disabled" {
			$l configure -background $Palette(disabledBackground)
		}
		"readonly" {
			$l configure -background $Palette(readonlyBackground)
		}
	}
}

######################################################################

set MenuEntryProc(create)	MenuEntry_Create
set MenuEntryProc(configure)	MenuEntry_Config
set MenuEntryProc(get)		MenuEntry_Get
set MenuEntryProc(delete)	MenuEntry_Delete
set MenuEntryProc(subwidget)	MenuEntry_SubWidget

proc MenuEntry { w args } {
	global MenuEntryProc
	return [$MenuEntryProc(create) $w $args]
}

proc MenuEntry_Proc { w a } {
	global MenuEntryProc
	set c [shift a]
	return [$MenuEntryProc($c) $w $a]
}

proc MenuEntry_Create { w a } {
	frame $w
	rename $w MenuEntry_$w

	entry $w.entry

	ListboxMenu $w.entry.menu
	button $w.button -image ImageDown -padx 0 -pady 0 -takefocus 0 \
		-command [list MenuEntry_ShowMenu $w]
	bind $w.entry <Key-Down> [list $w.entry.menu post]

	MenuEntry_SetState $w editable
	MenuEntry_Config $w $a

	pack $w.entry -side left -fill x -expand yes
	pack $w.button -side left

	proc $w { args } [list eval [join [list MenuEntry_Proc $w {$args}]]]
	return $w
}

proc MenuEntry_ShowMenu { w } {
	focus $w.entry
	$w.entry.menu post
}

proc MenuEntry_Config { w a } {
	set change_width 0

	while { [llength $a] > 0 } {
		set o [shift a]
		set v [shift a]

		if { $o == "-width" } {
			$w.entry configure -width $v
			set change_width 1
		} elseif { $o == "-height" } {
			$w.entry configure -height $v
		} elseif { $o == "-items" } {
			foreach i $v {
				$w.entry.menu add $i
			}
			set change_width 1
		} elseif { $o == "-variable" } {
			$w.entry configure -textvariable $v
			$w.entry.menu configure -variable $v
		} elseif { $o == "-state" } {
			MenuEntry_SetState $w $v
		}
	}
	if $change_width {
		set m 0
		foreach i [$w.entry.menu get 0 end] {
			set l [string length $i]
			if { $m < $l } {
				set m $l
			}
		}
		set l [$w.entry cget -width]
		if { $m < $l } {
			set m $l
		}
		$w.entry.menu configure -width $m
	}
}

proc MenuEntry_SetState { w s } {
	switch $s {
		"editable" {
			SetEntryState $w.entry normal
			$w.button configure -state normal
			bind $w.entry <ButtonRelease-1> {}
		}
		"buttononly" {
			SetEntryState $w.entry readonly
			$w.button configure -state normal
			bind $w.entry <ButtonRelease-1> \
				[list MenuEntry_ShowMenu $w]
		}
		"disabled" {
			SetEntryState $w.entry disabled
			$w.button configure -state disabled
			bind $w.entry <ButtonRelease-1> {}
		}
	}
}

proc MenuEntry_Get { w a } {
	$w.entry get
}

proc MenuEntry_Delete { w a } {
	eval $w.entry delete $a
}

proc MenuEntry_SubWidget { w a } {
	set r ""
	foreach i $a {
		if { $i == "entry" } {
			lappend r $w.entry
		} elseif { $i == "button" } {
			lappend r $w.button
		} elseif { $i == "menu" } {
			lappend r $w.entry.menu
		}
	}
	return $r
}

######################################################################

set UpDownEntryProc(create)	UpDownEntry_Create
set UpDownEntryProc(configure)	UpDownEntry_Config
set UpDownEntryProc(subwidget)	UpDownEntry_SubWidget

set UpDownEntryRepeat 0
set UpDownEntryRepeatInit  500
set UpDownEntryRepeatInterval 100

proc UpDownEntry { w args } {
	global UpDownEntryProc
	return [$UpDownEntryProc(create) $w $args]
}

proc UpDownEntry_Proc { w a } {
	global UpDownEntryProc
	set c [shift a]
	return [$UpDownEntryProc($c) $w $a]
}

proc UpDownEntry_Create { w a } {
	frame $w
	rename $w UpDownEntry_$w

	entry $w.entry
	frame $w.button

	button $w.button.up -image ImageMUp -padx 0 -pady 0 -takefocus 0
	button $w.button.down -image ImageMDown -padx 0 -pady 0 -takefocus 0

	bind $w.button.up   <Button-1> [list UpDownEntry_IncrStart $w 1]
	bind $w.button.down <Button-1> [list UpDownEntry_IncrStart $w -1]
	bind $w.button.up   <ButtonRelease-1> { set UpDownEntryRepeat 0 }
	bind $w.button.down <ButtonRelease-1> { set UpDownEntryRepeat 0 }

	bind $w.entry <Key-Up> \
		[list UpDownEntry_Incr $w 1 0]
	bind $w.entry <Key-Down> \
		[list UpDownEntry_Incr $w -1 0]

	global UpDownEntryConfig
	set UpDownEntryConfig($w,-minimum) {}
	set UpDownEntryConfig($w,-maximum) {}

	UpDownEntry_SetState $w editable
	UpDownEntry_Config $w $a

	pack $w.button.up $w.button.down -side top
	pack $w.entry -side left -fill x -expand yes
	pack $w.button -side left

	proc $w { args } [list eval [join [list UpDownEntry_Proc $w {$args}]]]
	return $w
}

proc UpDownEntry_Config { w a } {
	global UpDownEntryConfig UpDownEntryRepeatInit

	while { [llength $a] > 0 } {
		set o [shift a]
		set v [shift a]

		if { $o == "-width" } {
			$w.entry configure -width $v
		} elseif { $o == "-height" } {
			$w.entry configure -height $v
		} elseif { $o == "-variable" } {
			$w.entry configure -textvariable $v
		} elseif { $o == "-state" } {
			UpDownEntry_SetState $w $v
		} elseif { $o == "-upstep" } {
			bind $w.button.up   <Button-1> \
				[list UpDownEntry_IncrStart $w $v]
			bind $w.entry <Key-Up> \
				[list UpDownEntry_Incr $w $v]
		} elseif { $o == "-downstep" } {
			bind $w.button.down <Button-1> \
				[list UpDownEntry_IncrStart $w "-$v"]
			bind $w.entry <Key-Down> \
				[list UpDownEntry_Incr $w "-$v"]
		} elseif { $o == "-minimum" } {
			set UpDownEntryConfig($w,$o) $v
		} elseif { $o == "-maximum" } {
			set UpDownEntryConfig($w,$o) $v
		}
	}
}

proc UpDownEntry_SetState { w s } {
	switch $s {
		"editable" {
			SetEntryState $w.entry normal
			$w.button.up configure -state normal
			$w.button.down configure -state normal
		}
		"buttononly" {
			SetEntryState $w.entry readonly
			$w.button.up configure -state normal
			$w.button.down configure -state normal
		}
		"disabled" {
			SetEntryState $w.entry disabled
			$w.button.up configure -state disabled
			$w.button.down configure -state disabled
		}
	}
}

proc UpDownEntry_SubWidget { w a } {
	set r ""
	foreach i $a {
		if { $i == "entry" } {
			lappend r $w.entry
		} elseif { $i == "button" } {
			lappend r $w.button
		}
	}
	return $r
}

proc UpDownEntry_IncrStart { w step } {
	global UpDownEntryRepeat UpDownEntryRepeatInit

	UpDownEntry_Incr $w $step
	set UpDownEntryRepeat 1
	after $UpDownEntryRepeatInit \
		[list UpDownEntry_IncrRepeat $w $step]
}

proc UpDownEntry_IncrRepeat { w step } {
	global UpDownEntryRepeat UpDownEntryRepeatInterval

	if { $UpDownEntryRepeat } {
		UpDownEntry_Incr $w $step
		after $UpDownEntryRepeatInterval \
			[list UpDownEntry_IncrRepeat $w $step ]
	}
}

proc UpDownEntry_Incr { w step } {
	global UpDownEntryConfig

	set v [$w.entry get]
	if { [string length $v] == 0 } {
		$w.entry insert 0 0
	} else {
		incr v $step
		if { $step > 0 } {
			set m $UpDownEntryConfig($w,-maximum)
			if { [llength $m] <= 0 || $v <= $m } {
				$w.entry delete 0 end
				$w.entry insert 0 $v
			}
		} elseif { $step < 0 } {
			set m $UpDownEntryConfig($w,-minimum)
			if { [llength $m] <= 0 || $v >= $m } {
				$w.entry delete 0 end
				$w.entry insert 0 $v
			}
		}
	}
	return $v
}

######################################################################

set LabelFrameProc(create)	LabelFrame_Create
set LabelFrameProc(configure)	LabelFrame_Config
set LabelFrameProc(subwidget)	LabelFrame_SubWidget

proc LabelFrame { w args } {
	return [LabelFrame_Create $w $args]
}

proc LabelFrame_Proc { w a } {
	global LabelFrameProc
	set c [shift a]
	return [$LabelFrameProc($c) $w $a]
}

proc LabelFrame_Create { w a } {
	global Palette

	frame $w
	rename $w LableFrame_$w

	label $w.label -background $Palette(background)
	frame $w.frame -background $Palette(background)
	frame $w.border -relief groove -borderwidth 2 \
		-background $Palette(background)

	grid $w.label  -row 0 -rowspan 2 -column 2 -columnspan 1 -sticky n
	grid $w.frame  -row 3 -rowspan 1 -column 1 -columnspan 3 -sticky ewns
	grid $w.border -row 1 -rowspan 4 -column 0 -columnspan 5 -sticky ewns

	grid columnconfigure $w 0 -weight 0 -minsize 10
	grid columnconfigure $w 1 -weight 0
	grid columnconfigure $w 2 -weight 0
	grid columnconfigure $w 3 -weight 1
	grid columnconfigure $w 4 -weight 0 -minsize 10
	grid rowconfigure $w 0 -weight 0
	grid rowconfigure $w 1 -weight 0
	grid rowconfigure $w 2 -weight 0
	grid rowconfigure $w 3 -weight 1
	grid rowconfigure $w 4 -weight 0 -minsize 10

	raise $w.label $w.border
	raise $w.frame $w.border

	LabelFrame_Config $w $a

	proc $w { args } [list eval [join [list LabelFrame_Proc $w {$args}]]]
	return $w
}

proc LabelFrame_SetUpperPad { w p } {
	tkwait visibility $w.label
	set h [expr [winfo height $w.label]/2]
	if { $h < $p } {
		grid rowconfigure $w 2 -minsize $p
	}
	bind $w <Visibility> {}
}

proc LabelFrame_Config { w a } {
	while { [llength $a] > 0 } {
		set o [shift a]
		set v [shift a]
		LabelFrame_Config1 $w $o $v
	}
}

proc LabelFrame_Config1 { w o v } {
	switch -- $o {
		-label {
			$w.label configure -text $v
		}
		-background {
			$w.label configure -background $v
			$w.frame configure -background $v
			$w.border configure -background $v
		}
		-foreground {
			$w.label configure -foreground $v
		}
		-borderwidth {
			$w.border configure -borderwidth $v
		}
		-relief {
			$w.border configure -relief $v
		}
		-padx {
			grid columnconfigure $w 0 -minsize $v
			grid columnconfigure $w 4 -minsize $v
		}
		-pady {
			grid rowconfigure $w 4 -minsize $v
			bind $w <Visibility> \
				[list LabelFrame_SetUpperPad $w $v]
		}
		default {
		}
	}
}

proc LabelFrame_SubWidget { w a } {
	switch $a {
		frame	{ return $w.frame }
		label	{ return $w.label }
		border	{ return $w.border }
		default	{ return {} }
	}
}

######################################################################

set IpEntryProc(create)		IpEntry_Create
set IpEntryProc(configure)	IpEntry_Config
set IpEntryProc(get)		IpEntry_Get
set IpEntryProc(delete)		IpEntry_Delete

proc IpEntry { w args } {
	return [IpEntry_Create $w $args]
}

proc IpEntry_Proc { w a } {
	global IpEntryProc
	set c [shift a]
	return [$IpEntryProc($c) $w $a]
}

proc IpEntry_Create { w a } {
	global Palette
	global IpEntryEntries

	frame $w -relief sunken -borderwidth 2 -highlightthickness 1
	rename $w IpEntry_$w

	set IpEntryEntries($w) \
		[list \
			 [list $w.addr1 3 $w.dot1 . ] \
			 [list $w.addr2 3 $w.dot2 . ] \
			 [list $w.addr3 3 $w.dot3 . ] \
			 [list $w.addr4 3 $w.dot4 / ] \
			 [list $w.mask  2 ""      "" ] \
			]
	foreach e $IpEntryEntries($w) {
		set ent [lindex $e 0]
		set wth [lindex $e 1]
		set sep [lindex $e 2]
		set txt [lindex $e 3]

		entry $ent -borderwidth 0 -relief flat -justify right \
			-highlightthickness 0 -width $wth
		SetEntryState $ent normal
		bindtags $ent [list $ent IpEntry . all]
		pack $ent -side left -padx 0 -pady 0 -ipadx 0 -ipady 0

		if { $sep != "" } {
			label $sep -borderwidth 0 -relief flat -width 1 \
				-background $Palette(writableBackground) \
				-text $txt
			pack $sep -side left -padx 0 -pady 0 -ipadx 0 -ipady 0
		}
	}
	IpEntry_Config $w $a

	proc $w { args } [list eval [join [list IpEntry_Proc $w {$args}]]]
	return $w
}

proc IpEntry_Get { w a } {
	global IpEntryEntries

	set val {}
	foreach i $IpEntryEntries($w) {
		set v [[lindex $i 0] get]
		set d [lindex $i 3]
		set val "$val$v$d"
	}
	regsub {/$} $val {} val
	return $val
}

proc IpEntry_Delete { w a } {
	global IpEntryEntries

	foreach i $IpEntryEntries($w) {
		set e [lindex $i 0]
		$e delete 0 end
	}
}

proc IpEntry_Config { w a } {
	while { [llength $a] > 0 } {
		set o [shift a]
		set v [shift a]
		IpEntry_Config1 $w $o $v
	}
}

proc IpEntry_Config1 { w o v } {
	global IpEntryEntries
	switch -- $o {
		-state {
			foreach e $IpEntryEntries($w) {
				set ent [lindex $e 0]
				set sep [lindex $e 2]
				if { $ent != "" } {
					SetEntryState $ent $v
				}
				if { $sep != "" } {
					SetLabelState $sep $v
				}
			}
			foreach i { 1 2 3 4 } {
				SetEntryState $w.addr$i $v
			}
		}
		-variable {
			upvar \#0 $v var
			proc IpEntry_WriteTracer_$w { name elem op } \
				[list IpEntry_WriteTracer $w $v]
			trace variable var w IpEntry_WriteTracer_$w

			proc IpEntry_ReadTracer_$w { name elem op } \
				[list IpEntry_ReadTracer $w $v]
			trace variable var r IpEntry_ReadTracer_$w
		}
		-netmaskon {
			set ent [lindex [lindex $IpEntryEntries($w) 4] 0]
			set sep [lindex [lindex $IpEntryEntries($w) 3] 2]
			if { !$v } {
				$ent delete 0 end
				pack forget $ent
				pack forget $sep
			} else {
				pack $sep $ent -side left
			}
		}
	}
}

set IpEntryMaskTable {
	0xff 0xfe 0xfc 0xf8 0xf0 0xe0 0xc0 0x80 0x00
}

proc IpEntry_WriteTracer { win varname } {
	global IpEntryEntries
	upvar \#0 $varname var
	set l [split $var "/"]
	set addr [lindex $l 0]
	set mask [lindex $l 1]

	set octs [split $addr "."]
	for { set i 0 } { $i < 4 } { incr i } {
		set ent [lindex [lindex $IpEntryEntries($win) $i] 0]
		set state [$ent cget -state]
		$ent configure -state normal
		$ent delete 0 end
		$ent insert 0 [lindex $octs $i]
		$ent configure -state $state
	}
	set octs [split $mask "."]
	set ent [lindex [lindex $IpEntryEntries($win) 4] 0]

	set state [$ent cget -state]
	$ent configure -state normal
	$ent delete 0 end

	if { [llength $octs] == 1 } {
		$ent insert 0 $octs
	} elseif { [llength $octs] > 1 && [llength $octs] < 5 } {
		set m 0
		set i 0
		foreach o $octs {
			global IpEntryMaskTable
			set n 8
			foreach t $IpEntryMaskTable {
				if { $t == $o } {
					break
				}
				incr n -1
			}
			incr m $n
			incr i
		}
		$ent insert 0 $m
	}
		$ent configure -state $state
}

proc IpEntry_ReadTracer { win varname } {
	global IpEntryEntries
	upvar \#0 $varname var

	set octs {}
	for { set i 0 } { $i < 4 } { incr i } {
		set ent [lindex [lindex $IpEntryEntries($win) $i] 0]
		set octs [concat $octs [$ent get]]
	}
	set addr [join $octs "."]
	set ent [lindex [lindex $IpEntryEntries($win) 4] 0]
	set mask [$ent get]
	if { $mask != "" } {
		set var [join [list $addr $mask] "/"]
	} else {
		set var $addr
	}
}

foreach e [bind Entry] {
	bind IpEntry $e [bind Entry $e]
}

bind IpEntry <Key> {}
foreach n { 0 1 2 3 4 5 6 7 8 9 period } {
	bind IpEntry <Key-$n> { IpEntry_Insert %W %A }
}
bind IpEntry <Left> { IpEntry_MoveCursor %W -1 }
bind IpEntry <Right> { IpEntry_MoveCursor %W 1 }

proc IpEntry_Insert { w a } {
	global IpEntryEntries

	if { $a == "" } {
		return
	}
	set p [winfo parent $w]
	set n [lsearch $IpEntryEntries($p) "$w *"]
	set e [lindex $IpEntryEntries($p) $n]

	set sep [lindex $e 2]
	set txt [lindex $e 3]
	set next [lindex [lindex $IpEntryEntries($p) [expr $n + 1]] 0]
	if { $sep != "" && $a == $txt } {
		focus $next
		return
	}
	catch {
		set insert [$w index insert]
		if { [$w index sel.first] <= $insert &&
		     [$w index sel.last]  >= $insert } {
			$w delete sel.first sel.last
		}
	}
	$w insert insert $a
	while { [$w get] > 255 } {
		$w delete 0 1
	}
	return
}

proc IpEntry_MoveCursor { w step } {
	global IpEntryEntries

	set c [expr [$w index insert] + $step]
	set p [winfo parent $w]
	set n [lsearch $IpEntryEntries($p) "$w *"]
	if { $c >= 0 && $c <= [string length [$w get]] } {
		tkEntrySetCursor $w $c
	} elseif { $c < 0 && $n > 0 } {
		set next [lindex [lindex $IpEntryEntries($p) [expr $n - 1]] 0]
		focus $next
		tkEntrySetCursor $next end
	} elseif { $c > [string length [$w get]] &&
		   $n < [expr [llength $IpEntryEntries($p)] - 1] } {
		set next [lindex [lindex $IpEntryEntries($p) [expr $n + 1]] 0]
		focus $next
		tkEntrySetCursor $next 0
	}
}

######################################################################

set NoteBookProc(create)	NoteBook_Create
set NoteBookProc(configure)	NoteBook_Config
set NoteBookProc(add)		NoteBook_Add
set NoteBookProc(page)		NoteBook_Page
set NoteBookProc(pageframe)	NoteBook_PageFrame
set NoteBookProc(select)	NoteBook_Select
set NoteBookProc(unselect)	NoteBook_UnSelect

set NoteBookMaxPages 16

proc NoteBook { w args } {
	return [NoteBook_Create $w $args]
}

proc NoteBook_Proc { w a } {
	global NoteBookProc
	set c [shift a]
	return [$NoteBookProc($c) $w $a]
}

proc NoteBook_Create { w a } {
	global NoteBookPageId NoteBookMaxPages NoteBookConfig

	frame $w
	rename $w NoteBook_$w
	set NoteBookPageId($w) 0

	set NoteBookConfig($w,-borderwidth) 2

	frame $w.blank
	grid $w.blank -column 0 -columnspan [expr $NoteBookMaxPages*3 + 1] \
		-row 3 -rowspan 1 -sticky ewns

	grid rowconfigure $w 0 -weight 0 \
		-minsize $NoteBookConfig($w,-borderwidth)
	grid rowconfigure $w 1 -weight 1
	grid rowconfigure $w 2 -weight 0 \
		-minsize $NoteBookConfig($w,-borderwidth)
	grid rowconfigure $w 3 -weight 1

	grid columnconfigure $w  [expr $NoteBookMaxPages*3] -weight 10

	proc $w { args } [list eval [join [list NoteBook_Proc $w {$args}]]]
	return $w
}

proc NoteBook_Config1 { w o v } {
	switch -- $o {
		-borderwidth {
		}
	}
}

proc NoteBook_Config { w a } {
}

proc NoteBook_Add { w a } {
	global NoteBookPageId NoteBookMaxPages NoteBookConfig

	set i $NoteBookPageId($w)
	set bd $NoteBookConfig($w,-borderwidth)

	frame $w.relief$i -relief raised -borderwidth $bd
	button $w.tag$i -relief flat -borderwidth 0 -padx 0 -pady 0
	frame $w.mask$i -relief flat -borderwidth 0 -height $bd
	frame $w.page$i -relief raised -borderwidth $bd

	set c [expr $i*3]
	grid $w.relief$i -column [expr $c+0] -columnspan 3 -row 0 -rowspan 3 \
		-sticky ewns
	grid $w.tag$i    -column [expr $c+1] -columnspan 1 -row 1 -rowspan 1 \
		-sticky {}
	grid $w.mask$i   -column [expr $c+1] -columnspan 1 -row 2 -rowspan 1 \
		-sticky ewns
	grid $w.page$i -column 0 -columnspan [expr $NoteBookMaxPages*3 + 1] \
		-row 2 -rowspan 2 -sticky ewns

	grid columnconfigure $w [expr $c+0] -weight 0 -minsize $bd
	grid columnconfigure $w [expr $c+1] -weight 1
	grid columnconfigure $w [expr $c+2] -weight 0 -minsize $bd

	lower $w.relief$i $w.blank
	raise $w.tag$i  $w.relief$i
	raise $w.page$i $w.relief$i
	lower $w.mask$i $w.relief$i

	NoteBook_PageConfig $w $i $a

	$w.tag$i configure -command [list NoteBook_Select $w $i]

	incr NoteBookPageId($w)
	return $i
}

proc NoteBook_Page { w a } {
	set i [shift a]
	return $w.page$i
}

proc NoteBook_PageConfig { w n a } {
	while { [llength $a] > 0 } {
		set o [shift a]
		set v [shift a]
		switch -- $o {
			-tag {
				$w.tag$n configure -text $v
			}
		}
	}
}

proc NoteBook_UnSelect { w a } {
	set i [shift a]
	if { $i > 0 } {
		set j [expr $i - 1]
		lower $w.relief$i $w.relief$j
	}
	raise $w.tag$i    $w.relief$i
	lower $w.mask$i   $w.blank
	lower $w.page$i   $w.blank

	grid $w.relief$i -column [expr $i*3] -columnspan 3
	grid $w.mask$i -column [expr $i*3 + 1] -columnspan 1
}

proc NoteBook_Select { w a } {
	global NoteBookPageId

	for { set i 0 } { $i < $NoteBookPageId($w) } { incr i } {
		NoteBook_UnSelect $w $i
	}
	set i [shift a]
	if { $i > 0 } {
		set j [expr $i - 1]
		raise $w.relief$i $w.relief$j
	}
	raise $w.tag$i  $w.relief$i
	raise $w.page$i $w.blank
	raise $w.mask$i $w.page$i

	set c [expr $i*3]
	if { $c > 0 } {
		grid $w.relief$i -column [expr $c - 1] -columnspan 5
		grid $w.mask$i -column [expr $c] -columnspan 3
	} else {
		grid $w.relief$i -column $c -columnspan 4
		grid $w.mask$i -column [expr $c + 1] -columnspan 1
	}
}

######################################################################
